/***************************************************************************
 *            SessionManager.h
 *
 *  Copyright  2008  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#ifndef SESSIONMANAGER_H
#define SESSIONMANAGER_H

#include "libtorrent/entry.hpp"
#include "libtorrent/bencode.hpp"
#include "libtorrent/session.hpp"
#include "libtorrent/session_settings.hpp"
#include "libtorrent/fingerprint.hpp"
#include "libtorrent/intrusive_ptr_base.hpp"

#include <boost/thread/thread.hpp>
#include <boost/asio.hpp>

#include "TorrentsManager.h"
#include "IpFilterManager.h"
#include "Timer.h"
#include "Torrent.h"

namespace Lince
{

class queue_limit_reached {};

class SessionManager;
typedef boost::intrusive_ptr<SessionManager> SessionManagerPtr;

class SessionManager : public libtorrent::intrusive_ptr_base<SessionManager>,\
		public libtorrent::session
{
	// Queues
	int active_shared_torrents, active_downloading_torrents;

	// Total size;
	boost::int64_t currentDownloadingSize;
	boost::int64_t totalDownloadingSize, totalSharedSize;

	// Ip Filter
	Lince::IpFilterManagerPtr ip_filter;

	SessionManager();

	// Session Timer
	Timer time;

	// Session update thread
	boost::thread update_thread;

	// Session io_service
	boost::asio::io_service io;

	void session_update_manager();
	void check_alerts(const boost::system::error_code&, boost::asio::deadline_timer*);
	void check_scheduler(const boost::system::error_code&, boost::asio::deadline_timer*);
	void update_sizes(const boost::system::error_code&, boost::asio::deadline_timer*);

	void update_settings();
	void load_resume_data();
	void save_torrents_local_data();
	void add_torrent_to_session(Lince::torrent_localinfo &);

public:
	static SessionManagerPtr create();

	long get_session_elapsed_time() { return time.elapsed(); };

	// Settings management
	void update_ports();
	void update_encryption_support();
	void update_proxy_settings();
	void update_upnp();
	void update_lsd();
	void update_dht();

	// Ip Filter management
	void update_ip_filter();
	Lince::IpFilterManagerPtr get_ip_filter() { return ip_filter; };

	void add_torrent_to_session(const boost::filesystem::path &, const std::string &,
			const std::vector<int> &, std::vector<std::string> &, bool );

	void recheck_torrent(const torrentPosition &);

	// Torrents Management
	void remove_downloading_torrent(int, bool);
	void remove_shared_torrent(int);

	void save_torrents_resume_data();

	void set_upload_speed_session(int);
	void set_download_speed_session(int);

	void update_upload_session_speed();
	void update_download_session_speed();

	boost::int64_t get_current_downloading_size() {
		return currentDownloadingSize;
	}

	boost::int64_t get_total_downloading_size() {
		return totalDownloadingSize;
	}

	boost::int64_t get_total_shared_size() {
		return totalSharedSize;
	}

	virtual ~SessionManager();
};

} /* namespace */

#endif
